__author__ = 'Lukas Leufen'
__date__ = '2020-12-10'

from mlair.data_handler.data_handler_mixed_sampling import DataHandlerMixedSampling, \
    DataHandlerMixedSamplingSingleStation, DataHandlerMixedSamplingWithFilterSingleStation, \
    DataHandlerMixedSamplingWithFirFilterSingleStation, DataHandlerMixedSamplingWithFirFilter, \
    DataHandlerFirFilterSingleStation, DataHandlerMixedSamplingWithClimateFirFilterSingleStation, \
    DataHandlerMixedSamplingWithClimateFirFilter
from mlair.data_handler.data_handler_with_filter import DataHandlerFilter, DataHandlerFilterSingleStation, \
    DataHandlerClimateFirFilterSingleStation
from mlair.data_handler.data_handler_single_station import DataHandlerSingleStation
from mlair.data_handler.default_data_handler import DefaultDataHandler
from mlair.configuration.defaults import DEFAULT_INTERPOLATION_METHOD
from mlair.helpers.testing import get_all_args

import pytest
import mock


class TestDataHandlerMixedSampling:

    def test_data_handler(self):
        obj = object.__new__(DataHandlerMixedSampling)
        assert obj.data_handler.__qualname__ == DataHandlerMixedSamplingSingleStation.__qualname__

    def test_data_handler_transformation(self):
        obj = object.__new__(DataHandlerMixedSampling)
        assert obj.data_handler_transformation.__qualname__ == DataHandlerMixedSamplingSingleStation.__qualname__

    def test_requirements(self):
        reqs = get_all_args(DefaultDataHandler)
        obj = object.__new__(DataHandlerMixedSampling)
        assert sorted(obj.own_args()) == reqs
        reqs = get_all_args(DataHandlerSingleStation, remove="self")
        assert sorted(obj._requirements) == reqs
        reqs = get_all_args(DataHandlerSingleStation, DefaultDataHandler, remove=["self", "id_class"])
        assert sorted(obj.requirements()) == reqs


class TestDataHandlerMixedSamplingSingleStation:

    def test_requirements(self):
        reqs = get_all_args(DataHandlerSingleStation)
        obj = object.__new__(DataHandlerMixedSamplingSingleStation)
        assert sorted(obj.own_args()) == reqs
        reqs = get_all_args(DataHandlerSingleStation, remove="self")
        assert sorted(obj.requirements()) == reqs

    @mock.patch("mlair.data_handler.data_handler_single_station.DataHandlerSingleStation.setup_samples")
    def test_init(self, mock_super_init):
        obj = DataHandlerMixedSamplingSingleStation("first_arg", "second", {}, test=23, sampling="hourly",
                                                    interpolation_limit=(1, 10))
        assert obj.sampling == ("hourly", "hourly")
        assert obj.interpolation_limit == (1, 10)
        assert obj.interpolation_method == (DEFAULT_INTERPOLATION_METHOD, DEFAULT_INTERPOLATION_METHOD)

    @pytest.fixture
    def kwargs_dict(self):
        return {"test1": 2, "param_2": "string", "another": (10, 2)}

    def test_update_kwargs_single_to_tuple(self, kwargs_dict):
        obj = object.__new__(DataHandlerMixedSamplingSingleStation)
        obj.update_kwargs("test1", "23", kwargs_dict)
        assert kwargs_dict["test1"] == (2, 2)
        obj.update_kwargs("param_2", "23", kwargs_dict)
        assert kwargs_dict["param_2"] == ("string", "string")

    def test_update_kwargs_tuple(self, kwargs_dict):
        obj = object.__new__(DataHandlerMixedSamplingSingleStation)
        obj.update_kwargs("another", "23", kwargs_dict)
        assert kwargs_dict["another"] == (10, 2)

    def test_update_kwargs_default(self, kwargs_dict):
        obj = object.__new__(DataHandlerMixedSamplingSingleStation)
        obj.update_kwargs("not_existing", "23", kwargs_dict)
        assert kwargs_dict["not_existing"] == ("23", "23")
        obj.update_kwargs("also_new", (4, 2), kwargs_dict)
        assert kwargs_dict["also_new"] == (4, 2)

    def test_update_kwargs_assert_failure(self, kwargs_dict):
        obj = object.__new__(DataHandlerMixedSamplingSingleStation)
        with pytest.raises(AssertionError):
            obj.update_kwargs("error_too_long", (1, 2, 3), kwargs_dict)

    def test_setup_samples(self):
        pass

    def test_load_and_interpolate(self):
        pass

    def test_set_inputs_and_targets(self):
        pass

    def test_setup_data_path(self):
        pass


class TestDataHandlerMixedSamplingWithFilterSingleStation:

    def test_requirements(self):

        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerSingleStation)
        obj = object.__new__(DataHandlerMixedSamplingWithFilterSingleStation)
        assert sorted(obj.own_args()) == reqs
        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerSingleStation, remove="self")
        assert sorted(obj._requirements) == []
        assert sorted(obj.requirements()) == reqs


class TestDataHandlerMixedSamplingWithFirFilter:

    def test_requirements(self):
        reqs = get_all_args(DataHandlerFilter, DefaultDataHandler)
        obj = object.__new__(DataHandlerMixedSamplingWithFirFilter)
        assert sorted(obj.own_args()) == reqs
        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerSingleStation, DataHandlerFirFilterSingleStation,
                            remove=["self"])
        assert sorted(obj._requirements) == reqs
        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerSingleStation, DataHandlerFilter,
                            DataHandlerFirFilterSingleStation, DefaultDataHandler, remove=["self", "id_class"])
        assert sorted(obj.requirements()) == reqs


class TestDataHandlerMixedSamplingWithFirFilterSingleStation:

    def test_requirements(self):
        reqs = get_all_args(DataHandlerFirFilterSingleStation, DataHandlerFilterSingleStation, DataHandlerSingleStation)
        obj = object.__new__(DataHandlerMixedSamplingWithFirFilterSingleStation)
        assert sorted(obj.own_args()) == reqs
        assert sorted(obj._requirements) == []
        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerFirFilterSingleStation, DataHandlerSingleStation,
                            remove="self")
        assert sorted(obj.requirements()) == reqs


class TestDataHandlerMixedSamplingWithClimateFirFilter:

    def test_requirements(self):
        reqs = get_all_args(DataHandlerMixedSamplingWithClimateFirFilter, DataHandlerFilter, DefaultDataHandler)
        obj = object.__new__(DataHandlerMixedSamplingWithClimateFirFilter)
        assert sorted(obj.own_args()) == reqs
        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerClimateFirFilterSingleStation,
                            DataHandlerSingleStation, DataHandlerFirFilterSingleStation, remove=["self"])
        assert sorted(obj._requirements) == reqs
        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerClimateFirFilterSingleStation,
                            DataHandlerSingleStation, DataHandlerFilter, DataHandlerMixedSamplingWithClimateFirFilter,
                            DefaultDataHandler, DataHandlerFirFilterSingleStation, remove=["self", "id_class"])
        assert sorted(obj.requirements()) == reqs


class TestDataHandlerMixedSamplingWithClimateFirFilterSingleStation:

    def test_requirements(self):
        reqs = get_all_args(DataHandlerClimateFirFilterSingleStation, DataHandlerFirFilterSingleStation,
                            DataHandlerFilterSingleStation, DataHandlerSingleStation)
        obj = object.__new__(DataHandlerMixedSamplingWithClimateFirFilterSingleStation)
        assert sorted(obj.own_args()) == reqs
        assert sorted(obj._requirements) == []
        reqs = get_all_args(DataHandlerFilterSingleStation, DataHandlerFirFilterSingleStation, DataHandlerSingleStation,
                            DataHandlerClimateFirFilterSingleStation, remove="self")
        assert sorted(obj.requirements()) == reqs


# class TestDataHandlerSeparationOfScales:
#
#     def test_data_handler(self):
#         obj = object.__new__(DataHandlerSeparationOfScales)
#         assert obj.data_handler.__qualname__ == DataHandlerSeparationOfScalesSingleStation.__qualname__
#
#     def test_data_handler_transformation(self):
#         obj = object.__new__(DataHandlerSeparationOfScales)
#         assert obj.data_handler_transformation.__qualname__ == DataHandlerSeparationOfScalesSingleStation.__qualname__
#
#     def test_requirements(self):
#         reqs = get_all_args(DefaultDataHandler)
#         obj = object.__new__(DataHandlerSeparationOfScales)
#         assert sorted(obj.own_args()) == reqs
#
#         reqs = get_all_args(DataHandlerSeparationOfScalesSingleStation, DataHandlerKzFilterSingleStation,
#                             DataHandlerMixedSamplingWithKzFilterSingleStation,DataHandlerFilterSingleStation,
#                             DataHandlerSingleStation, remove=["self", "id_class"])
#         assert sorted(obj._requirements) == reqs
#         reqs = get_all_args(DataHandlerSeparationOfScalesSingleStation, DataHandlerKzFilterSingleStation,
#                             DataHandlerMixedSamplingWithKzFilterSingleStation,DataHandlerFilterSingleStation,
#                             DataHandlerSingleStation, DefaultDataHandler, remove=["self", "id_class"])
#         assert sorted(obj.requirements()) == reqs

#
# class TestDataHandlerSeparationOfScalesSingleStation:
#     pass

